#!/bin/sh
# -------------------------------------------------------------------------
# (C) Copyright Marxmeier Software AG 2008-2014
#
# Eloquence JDLG Plugin Interface
#
# Function: Create code signing CA to issue JAR certificates.
# -------------------------------------------------------------------------

# Configuration
# -------------

# Edit the values below as necessary.
#
# Notes:
# - CA_TITLE and ORGANIZATION must follow X.500 Distinguished Names syntax.
#   Comma should be avoided or must be backslash-escaped.
# - CA_ALIAS must follow file naming conventions. Avoid spaces and special
#   characters.

CA_TITLE="Eloquence JDLG Code Signing CA"
CA_ALIAS="jdlg-ca"
ORGANIZATION="Marxmeier Software AG"
COUNTRY="DE"
DAYS_VALID=1095

# Certificate Revocation List Distribution Point
# Java wants to verify that a certificate was not revoked.
# To support this, configure an URL where Java can fetch
# the $CA_ALIAS.crl.pem file, for example:
# CRL_DISTRIBUTION_URL="http://host/jdlg/$CA_ALIAS.crl.pem"

CRL_DISTRIBUTION_URL=""
CRL_DISTRIBUTION_URL="http://rhg-lx/jdlg/plugin/$CA_ALIAS.crl.pem"

# Commands
OPENSSL="openssl"
KEYTOOL="keytool"

# -----------------
# End Configuration

SUBJECT="/C=$COUNTRY/O=$ORGANIZATION/OU=$CA_TITLE"
CA_DIR="$CA_ALIAS"
SSL_CONF="$CA_DIR/openssl.cnf"
KEYSTORE="keystore.bin"
PASSWORDS_CONF="passwords.cnf"

cd "`dirname $0`"

if [ ! -r "$PASSWORDS_CONF" ]; then
  echo "Password configuration file not found."
  exit 2
fi
. "./$PASSWORDS_CONF"
if [ -z "$KEYSTORE_PASS" \
  -o -z "$KEY_PASS" \
  -o -z "$PRIVATE_KEY_PASS" \
  -o -z "$PKCS12_PASS" ]; then
  echo "KEYSTORE_PASS or KEY_PASS or PRIVATE_KEY_PASS or PKCS12_PASS not configured."
  exit 2
fi

rm -f "$KEYSTORE"
rm -rf "$CA_DIR"
mkdir -p \
  "$CA_DIR" \
  "$CA_DIR/certs" \
  "$CA_DIR/crl" \
  "$CA_DIR/newcerts" \
  "$CA_DIR/private"
chmod 700 "$CA_DIR/private"
touch "$CA_DIR/index.txt"
echo "01" > "$CA_DIR/serial"
echo "01" > "$CA_DIR/crlnumber"

cat > "$SSL_CONF" <<EOF.
[ ca ]
default_ca                      = $CA_ALIAS
[ $CA_ALIAS ]
dir                             = $CA_DIR
certs                           = \$dir/certs
crl_dir                         = \$dir/crl
database                        = \$dir/index.txt
unique_subject                  = no
new_certs_dir                   = \$dir/newcerts
certificate                     = \$dir/$CA_ALIAS.ca.pem
serial                          = \$dir/serial
crlnumber                       = \$dir/crlnumber
crl                             = \$dir/$CA_ALIAS.crl.pem
private_key                     = \$dir/private/$CA_ALIAS.key.pem
RANDFILE                        = \$dir/private/.rand
x509_extensions                 = usr_cert
name_opt                        = ca_default
cert_opt                        = ca_default
crl_extensions                  = crl_ext
default_days                    = $DAYS_VALID
default_crl_days                = 30
default_md                      = default
preserve                        = no
policy                          = policy_anything
[ req ]
default_bits                    = 2048
default_keyfile                 = $CA_ALIAS.key.pem
distinguished_name              = req_distinguished_name
attributes                      = req_attributes
x509_extensions                 = v3_ca
string_mask                     = utf8only
[ req_distinguished_name ]
countryName                     = Country Name (2 letter code)
countryName_default             = $COUNTRY
countryName_min                 = 2
countryName_max                 = 2
stateOrProvinceName             = State or Province Name (full name)
localityName                    = Locality Name (eg, city)
0.organizationName              = Organization Name (eg, company)
0.organizationName_default      = $ORGANIZATION
organizationalUnitName          = Organizational Unit Name (eg, section)
organizationalUnitName_default  = $CA_TITLE
commonName                      = Common Name (eg, YOUR name)
commonName_max                  = 64
emailAddress                    = Email Address
emailAddress_max                = 64
[ req_attributes ]
challengePassword               = A challenge password
challengePassword_min           = 4
challengePassword_max           = 20
unstructuredName                = An optional company name
[ policy_anything ]
countryName                     = optional
stateOrProvinceName             = optional
localityName                    = optional
organizationName                = optional
organizationalUnitName          = optional
commonName                      = optional
emailAddress                    = optional
[ v3_ca ]
basicConstraints                = CA:true
subjectKeyIdentifier            = hash
authorityKeyIdentifier          = keyid:always,issuer
[ crl_ext ]
authorityKeyIdentifier          = keyid:always
[ usr_cert ]
basicConstraints                = CA:false
subjectKeyIdentifier            = hash
authorityKeyIdentifier          = keyid,issuer
nsComment                       = "OpenSSL Generated Certificate"
EOF.
if [ -n "$CRL_DISTRIBUTION_URL" ]; then
   cat >> "$SSL_CONF" <<EOF.
crlDistributionPoints           = URI:$CRL_DISTRIBUTION_URL
EOF.
fi

# Generate CA.

"$OPENSSL" req -new \
  -batch -config "$SSL_CONF" \
  -subj "$SUBJECT" \
  -passout "pass:$PRIVATE_KEY_PASS" \
  -keyout "$CA_DIR/private/$CA_ALIAS.key.pem" \
  -out "$CA_DIR/$CA_ALIAS.req.pem" \
|| exit 1

"$OPENSSL" ca \
  -batch -config "$SSL_CONF" \
  -selfsign -extensions v3_ca \
  -passin "pass:$PRIVATE_KEY_PASS" \
  -out "$CA_DIR/$CA_ALIAS.ca.pem" \
  -infiles "$CA_DIR/$CA_ALIAS.req.pem" \
|| exit 1

rm "$CA_DIR/$CA_ALIAS.req.pem"

# Generate CRL.

"$OPENSSL" ca \
  -config "$SSL_CONF" \
  -passin "pass:$PRIVATE_KEY_PASS" \
  -gencrl -out "$CA_ALIAS.crl.pem" \
|| exit 1

# Import CA certificate,
# requires to be DER-formatted.

"$OPENSSL" x509 \
  -in "$CA_DIR/$CA_ALIAS.ca.pem" \
  -out "$CA_DIR/$CA_ALIAS.ca.der" \
  -outform DER \
|| exit 1

"$KEYTOOL" -importcert -noprompt \
  -alias "$CA_ALIAS" \
  -keystore "$KEYSTORE" \
  -storepass "$KEYSTORE_PASS" \
  -keypass "$KEY_PASS" \
  -file "$CA_DIR/$CA_ALIAS.ca.der" \
|| exit 1

rm "$CA_DIR/$CA_ALIAS.ca.der"

# Export CA certificate to pkcs12 container.

"$OPENSSL" pkcs12 -export \
  -passin "pass:$PRIVATE_KEY_PASS" \
  -passout "pass:$PKCS12_PASSWORD" \
  -in "$CA_DIR/$CA_ALIAS.ca.pem" \
  -inkey "$CA_DIR/private/$CA_ALIAS.key.pem" \
  -out "$CA_ALIAS.p12" \
  -name "$CA_TITLE" \
|| exit 1

echo
echo "Code signing CA successfully created in $CA_DIR/$CA_ALIAS.ca.pem,"
echo "imported to $KEYSTORE, exported to $CA_ALIAS.p12."
echo "Empty CRL created in $CA_ALIAS.crl.pem."
echo
